﻿using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.SearchModels;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AbstractSoftwareInstallationDatabaseImplement.Implements
{
    public class PackageStorage : IPackageStorage
    {
        public List<PackageViewModel> GetFullList()
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            return context.Packages
                .Include(x => x.Softwares)
                .ThenInclude(x => x.Software)
                .ToList()
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<PackageViewModel> GetFilteredList(PackageSearchModel model)
        {
            if (string.IsNullOrEmpty(model.PackageName))
            {
                return new();
            }
            using var context = new AbstractSoftwareInstallationDatabase();
            return context.Packages
                .Include(x => x.Softwares)
                .ThenInclude(x => x.Software)
                .Where(x => x.PackageName.Contains(model.PackageName))
                .ToList()
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public PackageViewModel? GetElement(PackageSearchModel model)
        {
            if (string.IsNullOrEmpty(model.PackageName) &&
            !model.Id.HasValue)
            {
                return null;
            }
            using var context = new AbstractSoftwareInstallationDatabase();
            return context.Packages
                .Include(x => x.Softwares)
                .ThenInclude(x => x.Software)
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.PackageName)
                        && x.PackageName == model.PackageName)
                    || (model.Id.HasValue && x.Id == model.Id))
            ?.GetViewModel;
        }

        public PackageViewModel? Insert(PackageBindingModel model)
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            var newPackage = Package.Create(context, model);
            if (newPackage == null)
            {
                return null;
            }
            context.Packages.Add(newPackage);
            context.SaveChanges();
            return newPackage.GetViewModel;
        }

        public PackageViewModel? Update(PackageBindingModel model)
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var Package = context.Packages.FirstOrDefault(rec => rec.Id == model.Id);
                if (Package == null)
                {
                    return null;
                }
                Package.Update(model);
                context.SaveChanges();
                Package.UpdateSoftwares(context, model);
                transaction.Commit();
                return Package.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public PackageViewModel? Delete(PackageBindingModel model)
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            var element = context.Packages
                .Include(x => x.Softwares)
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Packages.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
