﻿using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.SearchModels;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AbstractSoftwareInstallationDatabaseImplement.Implements
{
    public class SoftwareStorage : ISoftwareStorage
    {
        public List<SoftwareViewModel> GetFullList()
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            return context.Softwares
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<SoftwareViewModel> GetFilteredList(SoftwareSearchModel model)
        {
            if (string.IsNullOrEmpty(model.SoftwareName))
            {
                return new();
            }
            using var context = new AbstractSoftwareInstallationDatabase();
            return context.Softwares
                .Where(x => x.SoftwareName.Contains(model.SoftwareName))
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public SoftwareViewModel? GetElement(SoftwareSearchModel model)
        {
            if (string.IsNullOrEmpty(model.SoftwareName) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new AbstractSoftwareInstallationDatabase();
            return context.Softwares
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.SoftwareName)
                        && x.SoftwareName == model.SoftwareName)
                    || (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }

        public SoftwareViewModel? Insert(SoftwareBindingModel model)
        {
            var newSoftware = Software.Create(model);
            if (newSoftware == null)
            {
                return null;
            }
            using var context = new AbstractSoftwareInstallationDatabase();
            context.Softwares.Add(newSoftware);
            context.SaveChanges();
            return newSoftware.GetViewModel;
        }

        public SoftwareViewModel? Update(SoftwareBindingModel model)
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            var Software = context.Softwares.FirstOrDefault(x => x.Id == model.Id);
            if (Software == null)
            {
                return null;
            }
            Software.Update(model);
            context.SaveChanges();
            return Software.GetViewModel;
        }

        public SoftwareViewModel? Delete(SoftwareBindingModel model)
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            var element = context.Softwares.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Softwares.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
