﻿using AbstractPackageInstallationDatabaseImplement.Models;
using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationDatabaseImplement.Models;
using AbstractSoftwareInstallationDataModels;
using AbstractSoftwareInstallationDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AbstractSoftwareInstallationDatabaseImplement.Models
{
    public class Package : IPackageModel
    {
        public int Id { get; private set; }
        [Required]
        public string PackageName { get; private set; } = string.Empty;
        [Required]
        public double Price { get; private set; }

        private Dictionary<int, (ISoftwareModel, int)>? _packageSoftwares = null;

        [NotMapped]
        public Dictionary<int, (ISoftwareModel, int)> PackageSoftware
        {
            get
            {
                if (_packageSoftwares == null)
                {
                    _packageSoftwares = Softwares.ToDictionary(recPC => recPC.SoftwareId, recPC => (recPC.Software as ISoftwareModel, recPC.Count));
                }
                return _packageSoftwares;
            }
        }

        [ForeignKey("PackageId")]
        public virtual List<PackageSoftware> Softwares { get; set; } = new();
        [ForeignKey("PackageId")]
        public virtual List<Order> Orders { get; set; } = new();


        public static Package? Create(AbstractSoftwareInstallationDatabase context, PackageBindingModel model)
        {
            var Softwares = context.Softwares;

            return new Package()
            {
                Id = model.Id,
                PackageName = model.PackageName,
                Price = model.Price,
                Softwares = model.PackageSoftware.Select(x => new PackageSoftware
                {
                    Software = context.Softwares.First(y => y.Id == x.Key),
                    Count = x.Value.Item2
                }).ToList()
            };
        }

        public void Update(PackageBindingModel model)
        {
            PackageName = model.PackageName;
            Price = model.Price;
        }
        public PackageViewModel GetViewModel => new()
        {
            Id = Id,
            PackageName = PackageName,
            Price = Price,
            PackageSoftware = PackageSoftware
        };

        public void UpdateSoftwares(AbstractSoftwareInstallationDatabase context, PackageBindingModel model)
        {
            var packageSoftwares = context.PackageSoftwares
                .Where(rec => rec.PackageId == model.Id).ToList();
            if (packageSoftwares != null && packageSoftwares.Count > 0)
            {   // удалили те, которых нет в модели
                context.PackageSoftwares.RemoveRange(packageSoftwares.Where(rec => !model.PackageSoftware.ContainsKey(rec.SoftwareId)));
                context.SaveChanges();
                // обновили количество у существующих записей
                foreach (var updateSoftware in packageSoftwares)
                {
                    updateSoftware.Count = model.PackageSoftware[updateSoftware.SoftwareId].Item2;
                    model.PackageSoftware.Remove(updateSoftware.SoftwareId);
                }
                context.SaveChanges();
            }
            var package = context.Packages.First(x => x.Id == Id);
            foreach (var pc in model.PackageSoftware)
            {
                context.PackageSoftwares.Add(new PackageSoftware
                {
                    Package = package,
                    Software = context.Softwares.First(x => x.Id == pc.Key),
                    Count = pc.Value.Item2
                });
                context.SaveChanges();
            }
            _packageSoftwares = null;
        }

    }
}
