﻿using AbstractSoftwareInstallationFileImplement.Models;
using System.Xml.Linq;

namespace AbstractSoftwareInstallationFileImplement
{
    internal class DataFileSingleton
    {

        private static DataFileSingleton? instance;
        private readonly string SoftwareFileName = "Software.xml";
        private readonly string OrderFileName = "Order.xml";
        private readonly string PackageFileName = "Package.xml";
        public List<Software> Softwares { get; private set; }
        public List<Order> Orders { get; private set; }
        public List<Package> Packages { get; private set; }
        public static DataFileSingleton GetInstance()
        {
            if (instance == null)
            {
                instance = new DataFileSingleton();
            }
            return instance;
        }
        public void SaveSoftwares() => SaveData(Softwares, SoftwareFileName, "Softwares", x => x.GetXElement);
        public void SavePackages() => SaveData(Packages, PackageFileName, "Packages", x => x.GetXElement);
        public void SaveOrders() => SaveData(Orders, OrderFileName, "Orders", x => x.GetXElement);
        private DataFileSingleton()
        {
            Softwares = LoadData(SoftwareFileName, "Software", x => Software.Create(x)!)!;
            Packages = LoadData(PackageFileName, "Package", x => Package.Create(x)!)!;
            Orders = LoadData(OrderFileName, "Order", x => Order.Create(x)!)!;
        }
        private static List<T>? LoadData<T>(string filename, string xmlNodeName, Func<XElement, T> selectFunction)
        {
            if (File.Exists(filename))
            {
                return XDocument.Load(filename)?.Root?.Elements(xmlNodeName)?.Select(selectFunction)?.ToList();
            }
            return new List<T>();
        }

        private static void SaveData<T>(List<T> data, string filename, string xmlNodeName, Func<T, XElement> selectFunction)
        {
            if (data != null)
            {
                new XDocument(new XElement(xmlNodeName, data.Select(selectFunction).ToArray())).Save(filename);
            }
        }
    }
}
