﻿using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.SearchModels;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AbstractSoftwareInstallationFileImplement.Implements
{
    public class PackageStorage : IPackageStorage
    {
        private readonly DataFileSingleton source;

        public PackageStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public PackageViewModel? GetElement(PackageSearchModel model)
        {
            if (string.IsNullOrEmpty(model.PackageName) && !model.Id.HasValue)
            {
                return null;
            }
            return source.Packages
            .FirstOrDefault(x => 
            (!string.IsNullOrEmpty(model.PackageName) 
            && x.PackageName == model.PackageName) 
            || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public List<PackageViewModel> GetFilteredList(PackageSearchModel model)
        {
            if (string.IsNullOrEmpty(model.PackageName))
            {
                return new();
            }
            return source.Packages
                .Where(x => x.PackageName.Contains(model.PackageName))
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<PackageViewModel> GetFullList()
        {
            return source.Packages.Select(x => x.GetViewModel).ToList();
        }

        public PackageViewModel? Insert(PackageBindingModel model)
        {
            model.Id = source.Packages.Count > 0 ? source.Packages.Max(x => x.Id) + 1 : 1;
            var newPackage = Package.Create(model);
            if (newPackage == null)
            {
                return null;
            }
            source.Packages.Add(newPackage);
            source.SavePackages();
            return newPackage.GetViewModel;
        }

        public PackageViewModel? Update(PackageBindingModel model)
        {
            var Package = source.Packages.FirstOrDefault(x => x.Id == model.Id);
            if (Package == null)
            {
                return null;
            }
            Package.Update(model);
            source.SavePackages();
            return Package.GetViewModel;
        }
        public PackageViewModel? Delete(PackageBindingModel model)
        {
            var Package = source.Packages.FirstOrDefault(x => x.Id == model.Id);
            if (Package == null)
            {
                return null;
            }
            source.Packages.Remove(Package);
            source.SavePackages();
            return Package.GetViewModel;
        }
    }
}
