﻿using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.SearchModels;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationFileImplement.Models;

namespace AbstractSoftwareInstallationFileImplement.Implements
{
    public class SoftwareStorage : ISoftwareStorage
    {
        private readonly DataFileSingleton _source;
        public SoftwareStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }
        public List<SoftwareViewModel> GetFullList()
        {
            return _source.Softwares.Select(x => x.GetViewModel).ToList();
        }
        public List<SoftwareViewModel> GetFilteredList(SoftwareSearchModel
       model)
        {
            if (string.IsNullOrEmpty(model.SoftwareName))
            {
                return new();
            }
            return _source.Softwares.Where(x => x.SoftwareName.Contains(model.SoftwareName))
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public SoftwareViewModel? GetElement(SoftwareSearchModel model)
        {
            if (string.IsNullOrEmpty(model.SoftwareName) && !model.Id.HasValue)
            {
                return null;
            }
            return _source.Softwares.FirstOrDefault(x => 
            (!string.IsNullOrEmpty(model.SoftwareName) 
            && x.SoftwareName == model.SoftwareName) 
            || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public SoftwareViewModel? Insert(SoftwareBindingModel model)
        {
            model.Id = _source.Softwares.Count > 0 ? _source.Softwares.Max(x => x.Id) + 1 : 1;
            var newSoftware = Software.Create(model);
            if (newSoftware == null)
            {
                return null;
            }
            _source.Softwares.Add(newSoftware);
            _source.SaveSoftwares();
            return newSoftware.GetViewModel;
        }
        public SoftwareViewModel? Update(SoftwareBindingModel model)
        {
            var software = _source.Softwares.FirstOrDefault(x => x.Id == model.Id);
            if (software == null)
            {
                return null;
            }
            software.Update(model);
            _source.SaveSoftwares();
            return software.GetViewModel;
        }
        public SoftwareViewModel? Delete(SoftwareBindingModel model)
        {
            var software = _source.Softwares.FirstOrDefault(x => x.Id == model.Id);
            if (software == null)
            {
                return null;
            }
            _source.Softwares.Remove(software);
            _source.SaveSoftwares();
            return software.GetViewModel;
        }
    }
}
