﻿using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace AbstractSoftwareInstallationFileImplement.Models
{
    internal class Package : IPackageModel
    {
        public string PackageName { get; private set; } = string.Empty;

        public double Price { get; private set; }

        public int Id { get; private set; }

        public Dictionary<int, int> Softwares { get; private set; } = new();

        private Dictionary<int, (ISoftwareModel, int)>? _PackageSoftware = null;
        public Dictionary<int, (ISoftwareModel, int)> PackageSoftware
        {
            get
            {
                if (_PackageSoftware == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _PackageSoftware = Softwares.ToDictionary(
                        x => x.Key,
                        y => ((source.Softwares.FirstOrDefault(z => z.Id == y.Key) as ISoftwareModel)!, y.Value)
                    );
                }
                return _PackageSoftware;
            }
        }


        public static Package? Create(PackageBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Package()
            {
                Id = model.Id,
                PackageName = model.PackageName,
                Price = model.Price,
                Softwares = model.PackageSoftware.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }

        public static Package? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Package()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                PackageName = element.Element("PackageName")!.Value,
                Price = Convert.ToDouble(element.Element("Price")!.Value),
                Softwares = element.Element("PackageSoftware")!.Elements("PackageSoftware").ToDictionary(
                    x => Convert.ToInt32(x.Element("Key")?.Value),
                    x => Convert.ToInt32(x.Element("Value")?.Value)
                )
            };
        }


        public void Update(PackageBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            PackageName = model.PackageName;
            Price = model.Price;
            Softwares = model.PackageSoftware.ToDictionary(x => x.Key, x => x.Value.Item2);
            _PackageSoftware = null;
        }
        public PackageViewModel GetViewModel => new()
        {
            Id = Id,
            PackageName = PackageName,
            Price = Price,
            PackageSoftware = PackageSoftware
        };

        public XElement GetXElement => new(
            "Package",
             new XAttribute("Id", Id),
             new XElement("PackageName", PackageName),
             new XElement("Price", Price.ToString()),
             new XElement("PackageSoftware", Softwares.Select(x =>
             new XElement("PackageSoftware",
             new XElement("Key", x.Key),
             new XElement("Value", x.Value)))
            .ToArray()));


    }
}
