﻿using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace AbstractSoftwareInstallationFileImplement.Models
{
    internal class Software : ISoftwareModel
    {
        public int Id { get; private set; }
        public string SoftwareName { get; private set; } = string.Empty;
        public double Cost { get; set; }
        public static Software? Create(SoftwareBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Software()
            {
                Id = model.Id,
                SoftwareName = model.SoftwareName,
                Cost = model.Cost
            };
        }
        public static Software? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Software()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                SoftwareName = element.Element("SoftwareName")!.Value,
                Cost = Convert.ToDouble(element.Element("Cost")!.Value)
            };
        }
        public void Update(SoftwareBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            SoftwareName = model.SoftwareName;
            Cost = model.Cost;
        }
        public SoftwareViewModel GetViewModel => new()
        {
            Id = Id,
            SoftwareName = SoftwareName,
            Cost = Cost
        };
        public XElement GetXElement => new("Software",
            new XAttribute("Id", Id),
            new XElement("SoftwareName", SoftwareName),
            new XElement("Cost", Cost.ToString()));
    }
}
