﻿using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.SearchModels;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationListImplement.Models;

namespace AbstractSoftwareInstallationListImplement.Implements
{
    public class SoftwareStorage : ISoftwareStorage
    {
        private readonly DataListSingleton _source;
        public SoftwareStorage()
        {
            _source = DataListSingleton.GetInstance();
        }
        public List<SoftwareViewModel> GetFullList()
        {
            var result = new List<SoftwareViewModel>();
            foreach (var component in _source.Softwares)
            {
                result.Add(component.GetViewModel);
            }
            return result;
        }
        public List<SoftwareViewModel> GetFilteredList(SoftwareSearchModel
       model)
        {
            var result = new List<SoftwareViewModel>();
            if (string.IsNullOrEmpty(model.SoftwareName))
            {
                return result;
            }
            foreach (var component in _source.Softwares)
            {
                if (component.SoftwareName.Contains(model.SoftwareName))
                {
                    result.Add(component.GetViewModel);
                }
            }
            return result;
        }
        public SoftwareViewModel? GetElement(SoftwareSearchModel model)
        {
            if (string.IsNullOrEmpty(model.SoftwareName) && !model.Id.HasValue)
            {
                return null;
            }
        foreach (var software in _source.Softwares)
            {
                if ((!string.IsNullOrEmpty(model.SoftwareName) &&
               software.SoftwareName == model.SoftwareName) ||
                (model.Id.HasValue && software.Id == model.Id))
                {
                    return software.GetViewModel;
                }
            }
            return null;
        }
        public SoftwareViewModel? Insert(SoftwareBindingModel model)
        {
            model.Id = 1;
            foreach (var software in _source.Softwares)
            {
                if (model.Id <= software.Id)
                {
                    model.Id = software.Id + 1;
                }
            }
            var newSoftware = Software.Create(model);
            if (newSoftware == null)
            {
                return null;
            }
            _source.Softwares.Add(newSoftware);
            return newSoftware.GetViewModel;
        }
        public SoftwareViewModel? Update(SoftwareBindingModel model)
        {
            foreach (var component in _source.Softwares)
            {
                if (component.Id == model.Id)
                {
                    component.Update(model);
                    return component.GetViewModel;
                }
                throw new ArgumentNullException("Новая цена", nameof(model.Cost));
            }
            return null;
        }
        public SoftwareViewModel? Delete(SoftwareBindingModel model)
        {
            for (int i = 0; i < _source.Softwares.Count; ++i)
            {
                if (_source.Softwares[i].Id == model.Id)
                {
                    var element = _source.Softwares[i];
                    _source.Softwares.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }

    }
}
